import os.path
import unittest

import umu_commander.configuration as config
from tests import *
from umu_commander import configuration


class Config(unittest.TestCase):
    def setUp(self):
        configuration.CONFIG_DIR = TESTING_DIR
        configuration.DB_DIR = TESTING_DIR
        setup()

    def tearDown(self):
        teardown()

    def test_missing_config(self):
        config.dump()
        self.assertTrue(
            os.path.exists(os.path.join(TESTING_DIR, configuration.CONFIG_NAME))
        )
        config.load()



import os
import shutil

import umu_commander.database as db
from umu_commander.classes import ProtonDir, ProtonVer
from umu_commander.proton import (
    collect_proton_versions,
    get_latest_umu_proton,
    refresh_proton_versions,
)
from umu_commander.util import (
    get_selection,
)


def untrack(quiet: bool = False):
    current_dir: str = os.getcwd()
    for proton_dir in db.get().keys():
        for proton_ver in db.get(proton_dir):
            if current_dir in db.get(proton_dir, proton_ver):
                db.get(proton_dir, proton_ver).remove(current_dir)

    if not quiet:
        print("Directory removed from all tracking lists.")


def track(
    proton_ver: ProtonVer = None, refresh_versions: bool = True, quiet: bool = False
):
    if refresh_versions:
        refresh_proton_versions()

    if proton_ver is None:
        proton_ver: ProtonVer = get_selection(
            "Select Proton version to track directory with:",
            None,
            collect_proton_versions(sort=True),
        ).as_proton_ver()

    untrack(quiet=True)
    current_directory: str = os.getcwd()
    db.get(proton_ver.dir, proton_ver.version_num).append(current_directory)

    if not quiet:
        print(
            f"Directory {current_directory} added to Proton version's {proton_ver.version_num} in {proton_ver.dir} tracking list."
        )


def users():
    proton_dirs: list[ProtonDir] = collect_proton_versions(sort=True, user_count=True)

    proton_ver: ProtonVer = get_selection(
        "Select Proton version to view user list:", None, proton_dirs
    ).as_proton_ver()

    if proton_ver.dir in db.get() and proton_ver.version_num in db.get(proton_ver.dir):
        version_users: list[str] = db.get(proton_ver.dir, proton_ver.version_num)
        if len(version_users) > 0:
            print(
                f"Directories tracked by {proton_ver.version_num} of {proton_ver.dir}:",
                *version_users,
                sep="\n\t",
            )

        else:
            print("This version is tracking no directories.")

    else:
        print("This version hasn't been used by umu before.")


def delete():
    for proton_dir in db.get().keys():
        for proton_ver, version_users in db.get(proton_dir).copy().items():
            if proton_ver == get_latest_umu_proton():
                continue

            if len(version_users) == 0:
                selection: str = input(
                    f"Version {proton_ver} in {proton_dir} is tracking no directories, delete? (Y/N) ? "
                )
                if selection.lower() == "y":
                    try:
                        shutil.rmtree(os.path.join(proton_dir, proton_ver))
                    except FileNotFoundError:
                        pass
                    del db.get(proton_dir)[proton_ver]


def untrack_unlinked():
    for proton_dir in db.get().keys():
        for proton_ver, version_users in db.get()[proton_dir].items():
            for user in version_users:
                if not os.path.exists(user):
                    db.get(proton_dir, proton_ver).remove(user)



import os
import sys
from collections.abc import Callable
from json import JSONDecodeError

from umu_commander import configuration as config
from umu_commander import database as db
from umu_commander import tracking, umu_config
from umu_commander.classes import ExitCode
from umu_commander.configuration import CONFIG_DIR, CONFIG_NAME
from umu_commander.util import print_help


def main() -> ExitCode:
    try:
        config.load()
    except (JSONDecodeError, KeyError):
        config_path: str = os.path.join(CONFIG_DIR, CONFIG_NAME)
        old_config_path: str = os.path.join(CONFIG_DIR, CONFIG_NAME + ".old")

        print(f"Config file at {config_path} could not be read.")

        if not os.path.exists(old_config_path):
            print(f"Config file renamed to {old_config_path}.")
            os.rename(config_path, old_config_path)

    except FileNotFoundError:
        config.dump()

    try:
        db.load()
    except JSONDecodeError:
        db_path: str = os.path.join(config.DB_DIR, config.DB_NAME)
        old_db_path: str = os.path.join(config.DB_DIR, config.DB_NAME + ".old")

        print(f"Tracking file at {db_path} could not be read.")

        if not os.path.exists(old_db_path):
            print(f"DB file renamed to {old_db_path}.")
            os.rename(db_path, old_db_path)

    except FileNotFoundError:
        pass

    dispatch: dict[str, Callable] = {
        "track": tracking.track,
        "untrack": tracking.untrack,
        "users": tracking.users,
        "delete": tracking.delete,
        "create": umu_config.create,
        "run": umu_config.run,
    }

    if len(sys.argv) == 1:
        print_help()
        return ExitCode.SUCCESS.value
    elif sys.argv[1] not in dispatch:
        print("Invalid verb.")
        print_help()
        return ExitCode.INVALID_SELECTION.value

    dispatch[sys.argv[1]]()

    tracking.untrack_unlinked()
    db.dump()

    return ExitCode.SUCCESS.value


if __name__ == "__main__":
    exit(main())



from dataclasses import dataclass
from enum import Enum


@dataclass
class Element:
    group_id: str = ""
    value: str = ""
    info: str = ""

    def as_proton_ver(self) -> "ProtonVer":
        return ProtonVer(self.group_id, self.value, self.info)

    def as_dll_override(self) -> "DLLOverride":
        return DLLOverride(self.info, self.value)


@dataclass
class ProtonVer(Element):
    def __init__(self, dir: str = "", version_num: str = "", user_count: str = ""):
        super().__init__(group_id=dir, value=version_num, info=user_count)

    @property
    def dir(self):
        return self.group_id

    @property
    def version_num(self):
        return self.value

    @property
    def user_count(self):
        return self.info


@dataclass
class DLLOverride(Element):
    def __init__(self, label: str = "", override_str: str = ""):
        super().__init__(group_id="", value=override_str, info=label)

    @property
    def override_str(self):
        return self.value

    @property
    def label(self):
        return self.info


@dataclass
class Value(Element):
    def __init__(self, value: str):
        super().__init__(value=value)


@dataclass
class Group:
    identity: str = ""
    label: str = ""
    elements: list[Element] = list


@dataclass
class ProtonDir(Group):
    @property
    def path(self):
        return self.identity

    @property
    def versions(self) -> list[ProtonVer]:
        return [e.as_proton_ver() for e in self.elements]

    @versions.setter
    def versions(self, value):
        self.elements = value


class ExitCode(Enum):
    SUCCESS = 0
    DECODING_ERROR = 1
    INVALID_SELECTION = 2



import os
import subprocess
import tomllib
from typing import Any

import tomli_w

import umu_commander.configuration as config
from umu_commander import tracking
from umu_commander.classes import DLLOverride, ProtonVer
from umu_commander.proton import collect_proton_versions, refresh_proton_versions
from umu_commander.util import (
    get_selection,
    strings_to_values,
)


def create():
    refresh_proton_versions()

    params: dict[str, Any] = {"umu": {}, "env": {}}

    # Prefix selection
    selection: str = get_selection(
        "Select wine prefix:",
        strings_to_values(
            [*os.listdir(config.DEFAULT_PREFIX_DIR), "Current directory"]
        ),
        None,
    ).value

    if selection == "Current directory":
        params["umu"]["prefix"] = os.path.join(os.getcwd(), "prefix")
    else:
        params["umu"]["prefix"] = os.path.join(config.DEFAULT_PREFIX_DIR, selection)

    # Proton selection
    selected_umu_latest: bool = False
    proton_ver: ProtonVer = get_selection(
        "Select Proton version:",
        None,
        collect_proton_versions(sort=True),
    ).as_proton_ver()
    params["umu"]["proton"] = os.path.join(proton_ver.dir, proton_ver.version_num)

    # Select DLL overrides
    possible_overrides: list[DLLOverride] = [
        DLLOverride(label="Reset"),
        DLLOverride(label="Done"),
        *config.DLL_OVERRIDES_OPTIONS,
    ]
    selected: set[int] = set()
    while True:
        print("Select DLLs to override, multiple can be selected:")
        for idx, override in enumerate(possible_overrides):
            if idx in selected:
                idx = "Y"
            print(f"{idx}) {override.label}")

        try:
            index: int = int(input("? "))
            print("")
        except ValueError:
            continue

        if index == 0:
            selected = set()
            continue

        if index == 1:
            break

        index: int = int(index)
        if index - 1 < len(possible_overrides):
            selected.add(index)

    if len(selected) > 0:
        params["env"]["WINEDLLOVERRIDES"] = ""
        for selection in selected:
            # noinspection PyTypeChecker
            params["env"]["WINEDLLOVERRIDES"] += possible_overrides[
                selection
            ].override_str

    # Set language locale
    match get_selection(
        "Select locale:",
        strings_to_values(["Default", "Japanese"]),
        None,
    ).value:
        case "Default":
            pass
        case "Japanese":
            params["env"]["LANG"] = "ja_JP.UTF8"

    # Input executable launch args
    launch_args: list[str] = input(
        "Enter executable options, separated by spaces:\n? "
    ).split()
    params["umu"]["launch_args"] = launch_args

    # Select executable name
    files: list[str] = [
        file
        for file in os.listdir(os.getcwd())
        if os.path.isfile(os.path.join(os.getcwd(), file))
    ]
    executable_name: str = get_selection(
        "Select game executable:", strings_to_values(files), None
    ).value
    params["umu"]["exe"] = executable_name

    try:
        with open(config.UMU_CONFIG_NAME, "wb") as file:
            tomli_w.dump(params, file)

        print(f"Configuration file {config.UMU_CONFIG_NAME} created at {os.getcwd()}.")
        print(f"Use by running umu-commander run.")
        tracking.track(proton_ver, False)
    except:
        print("Could not create configuration file.")


def run():
    if not os.path.exists(config.UMU_CONFIG_NAME):
        print("No umu config in current directory.")
        return

    with open(config.UMU_CONFIG_NAME, "rb") as toml_file:
        toml_conf = tomllib.load(toml_file)

        if not os.path.exists(toml_conf["umu"]["prefix"]):
            os.mkdir(toml_conf["umu"]["prefix"])

        os.environ.update(toml_conf.get("env", {}))
        subprocess.run(
            args=["umu-run", "--config", config.UMU_CONFIG_NAME],
            env=os.environ,
        )
